pub struct AmgTransferOperators {
pub prolongation: CsrMatrix<S>,
pub restriction: CsrMatrix<S>,
}Available on crate feature
backend-faer only.Fields
prolongation: CsrMatrix<S>restriction: CsrMatrix<S>Implementations
Sourceimpl AmgTransferOperators
impl AmgTransferOperators
Sourcepub fn from_prolongation_adjoint(prolongation: CsrMatrix<S>) -> Self
pub fn from_prolongation_adjoint(prolongation: CsrMatrix<S>) -> Self
Build transfer operators with the canonical AMG restriction R = P^H.
AMG setup derives its restriction from the prolongation in scalar-generic Galerkin paths. This constructor makes that contract explicit for callers that only need to provide a custom prolongation.